package net.wanji.opt.controller.induce;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Maps;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.i18n.I18nResourceBundle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.framework.rest.Page;
import net.wanji.common.framework.rest.ValidationGroups;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.mapper.GreenwaveInfoMapper;
import net.wanji.opt.dao.mapper.induce.GreenwaveInducesMapper;
import net.wanji.opt.dto.induce.GreenwaveDTO;
import net.wanji.opt.dto.induce.GreenwaveInducesDTO;
import net.wanji.opt.dto.induce.InduceDTO;
import net.wanji.opt.entity.DeviceInduces;
import net.wanji.opt.entity.GreenwaveInduces;
import net.wanji.opt.entity.InduceTemplate;
import net.wanji.opt.service.DeviceInducesService;
import net.wanji.opt.service.induce.GreenwaveInducesHistService;
import net.wanji.opt.service.induce.GreenwaveInducesService;
import net.wanji.opt.service.induce.InduceTemplateService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 绿波-诱导屏幕关系
 * </p>
 *
 * @author guoliang.dong
 * @since 2024-11-22
 */
@Api(value = "GreenwaveInducesController", description = "绿波-诱导屏幕关系")
@RestController
@RequestMapping("/greenwaveInduces")
@Slf4j
public class GreenwaveInducesController {

    @Resource
    private GreenwaveInducesService greenwaveInducesService;

    @Resource
    private GreenwaveInducesMapper greenwaveInducesMapper;


//    @Resource
//    private InduceTemplateService induceTemplateService;

    @Resource
    private DeviceInducesService deviceInducesService;

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;

    @Resource
    private GreenwaveInducesHistService greenwaveInducesHistService;

    /**
     * 获取所有记录
     *
     * @return JsonViewObject
     */
    @ApiOperation(value = "获取所有记录", notes = "获取所有记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/byAll", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getAll() {
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start = System.currentTimeMillis();

        try {
//            List<GreenwaveInduces> list = this.greenwaveInducesService.list();
//            List<InduceTemplate> induceTemplateList = induceTemplateService.list();
            List<DeviceInduces> deviceInducesList = deviceInducesService.list();
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();

            List<InduceDTO> list = new ArrayList<>();
            this.greenwaveInducesService.list().stream().forEach(o -> {
                InduceDTO induceDTO = new InduceDTO();
                induceDTO.setId(o.getId());
                induceDTO.setEquipCode(o.getEquipCode());
                //获取关联设备信息
                List<DeviceInduces> deviceInduces = deviceInducesList.stream().filter(x -> Objects.nonNull(x.getEquipCode()) && x.getEquipCode().equals(o.getEquipCode())).collect(Collectors.toList());// 过滤条件// 收集结果
                if (Objects.nonNull(deviceInduces) && deviceInduces.size() > 0) {
                    induceDTO.setEquipName(deviceInduces.get(0).getEquipName());
                    induceDTO.setRidDir(o.getDir());
                    induceDTO.setResolutionWidth(deviceInduces.get(0).getResolutionWidth());
                    induceDTO.setResolutionHeight(deviceInduces.get(0).getResolutionHeight());
                    induceDTO.setWkt(deviceInduces.get(0).getWkt());
                }
                //获取上屏状态
                induceDTO.setStatus(o.getStatus());
                //获取关联绿波信息
                List<GreenwaveDTO> greenwaveList = new ArrayList<>();
                this.greenwaveInducesHistService.list().stream().filter(y ->Objects.nonNull(y.getGreenId()) && y.getGreenId().equals(o.getGreenId())).forEach(m -> {
                    GreenwaveDTO greenwaveDTO = new GreenwaveDTO();
                    greenwaveDTO.setGreenId(m.getGreenId());
                    //获取绿波基本信息
                    List<GreenwaveInfoPO> greenwaveInfoByGreenId = greenwaveInfoPOList.stream().filter(n -> n.getId().equals(o.getGreenId())).collect(Collectors.toList());// 过滤条件// 收集结果
                    if (Objects.nonNull(greenwaveInfoByGreenId) && greenwaveInfoByGreenId.size() > 0) {
                        greenwaveDTO.setGreenName(greenwaveInfoByGreenId.get(0).getName());
                        greenwaveDTO.setDesignSpeed(greenwaveInfoByGreenId.get(0).getDesignSpeed());
                    }
                    greenwaveDTO.setStrategyId(m.getStrategyId());
                    greenwaveDTO.setStrategyName(m.getStrategyName());
                    greenwaveDTO.setDir(m.getDir());
                    if(Objects.nonNull(m.getControlOptTimes()) && m.getControlOptTimes().split("\\|").length>0) {
                        try {
                            String startTime = DateUtil.format(DateUtil.parse(m.getControlOptTimes().split("\\|")[0], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                            String endTime = DateUtil.format(DateUtil.parse(m.getControlOptTimes().split("\\|")[1], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                            greenwaveDTO.setControlOptTimes(startTime.substring(0,startTime.lastIndexOf(":"))+"-"+endTime.substring(0,endTime.lastIndexOf(":")));
                        }catch (ParseException ex){ex.printStackTrace();
                            greenwaveDTO.setControlOptTimes(m.getControlOptTimes());
                        }catch (StringIndexOutOfBoundsException ex){ex.printStackTrace();
                            greenwaveDTO.setControlOptTimes(m.getControlOptTimes());
                        }
                    }else {
                        greenwaveDTO.setControlOptTimes(m.getControlOptTimes());
                    }
                    greenwaveList.add(greenwaveDTO);
                });
                induceDTO.setGreenwaveList(greenwaveList);
                //获取关联模板信息
                //induceDTO.setInduceTemplateList(induceTemplateList.stream().filter(z -> Objects.isNull(z.getEquipCode()) || z.getEquipCode().equals(o.getEquipCode())).collect(Collectors.toList()));
                list.add(induceDTO);
            });
            jsonView.success(list);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getAll error", this.getClass().getSimpleName(), e);
        }
        return jsonView;
    }

    /**
     * 根据条件查询记录
     *
     * @param greenwaveInduces 查询条件
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据条件查询记录", notes = "根据条件查询记录", response = JsonViewObject.class, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/byCondition", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject getByWhere(@ApiParam(value = "查询条件", required = true) @RequestBody @Validated({ValidationGroups.Query.class}) GreenwaveInduces greenwaveInduces){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String jsonStr = JSON.toJSONString(greenwaveInduces);
        try {
            //参数校验过程中修改，兼容实现，restful入参不用map，便于进行参数逐个校验
            Map params = JSONObject.parseObject(JSONObject.toJSONString(greenwaveInduces), Map.class);
//            Map<String, Object> params = new HashMap<>();
//            JSONObject.parseObject(JSONObject.toJSONString(GreenwaveInduces), Map.class).forEach((k,v) -> params.put(StringUtils.camelToCapital(k.toString()).toLowerCase(), v));
            List<GreenwaveInduces> greenwaveInducesList = this.greenwaveInducesMapper.findByMap(params);
//            List<InduceTemplate> induceTemplateList = induceTemplateService.list();
            List<DeviceInduces> deviceInducesList = deviceInducesService.list();
            List<GreenwaveInfoPO> greenwaveInfoPOList = greenwaveInfoMapper.selectAll();

            List<InduceDTO> list = new ArrayList<>();
            greenwaveInducesList.stream().forEach(o -> {
                InduceDTO induceDTO = new InduceDTO();
                induceDTO.setId(o.getId());
                induceDTO.setEquipCode(o.getEquipCode());
                //获取关联设备信息
                List<DeviceInduces> deviceInduces = deviceInducesList.stream().filter(x -> Objects.nonNull(x.getEquipCode()) && x.getEquipCode().equals(o.getEquipCode())).collect(Collectors.toList());// 过滤条件// 收集结果
                if (Objects.nonNull(deviceInduces) && deviceInduces.size() > 0) {
                    induceDTO.setEquipName(deviceInduces.get(0).getEquipName());
                    induceDTO.setRidDir(o.getDir());
                    induceDTO.setResolutionWidth(deviceInduces.get(0).getResolutionWidth());
                    induceDTO.setResolutionHeight(deviceInduces.get(0).getResolutionHeight());
                    induceDTO.setWkt(deviceInduces.get(0).getWkt());
                }
                //获取上屏状态
                induceDTO.setStatus(o.getStatus());
                //获取关联绿波信息
                List<GreenwaveDTO> greenwaveList = new ArrayList<>();
                this.greenwaveInducesHistService.list().stream().filter(y -> Objects.nonNull(y.getGreenId()) && y.getGreenId().equals(o.getGreenId())).forEach(m -> {
                    GreenwaveDTO greenwaveDTO = new GreenwaveDTO();
                    greenwaveDTO.setGreenId(m.getGreenId());
                    //获取绿波基本信息
                    List<GreenwaveInfoPO> greenwaveInfoByGreenId = greenwaveInfoPOList.stream().filter(n -> n.getId().equals(o.getGreenId())).collect(Collectors.toList());// 过滤条件// 收集结果
                    if (Objects.nonNull(greenwaveInfoByGreenId) && greenwaveInfoByGreenId.size() > 0) {
                        greenwaveDTO.setGreenName(greenwaveInfoByGreenId.get(0).getName());
                        greenwaveDTO.setDesignSpeed(greenwaveInfoByGreenId.get(0).getDesignSpeed());
                    }
                    greenwaveDTO.setStrategyId(m.getStrategyId());
                    greenwaveDTO.setStrategyName(m.getStrategyName());
                    greenwaveDTO.setDir(m.getDir());
                    Double maxSpeed = m.getMaxSpeed();
                    Double minSpeed = m.getMinSpeed();
                    greenwaveDTO.setMaxSpeed(maxSpeed.intValue());
                    greenwaveDTO.setMinSpeed(minSpeed.intValue());
                    if(Objects.nonNull(m.getControlOptTimes()) && m.getControlOptTimes().split("\\|").length>0) {
                        try {
                            String startTime = DateUtil.format(DateUtil.parse(m.getControlOptTimes().split("\\|")[0], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                            String endTime = DateUtil.format(DateUtil.parse(m.getControlOptTimes().split("\\|")[1], Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND), Constants.DATE_FORMAT.E_DATE_FORMAT_TIME);
                            greenwaveDTO.setControlOptTimes(startTime.substring(0,startTime.lastIndexOf(":"))+"-"+endTime.substring(0,endTime.lastIndexOf(":")));
                        }catch (ParseException ex){ex.printStackTrace();
                            greenwaveDTO.setControlOptTimes(m.getControlOptTimes());
                        }catch (StringIndexOutOfBoundsException ex){ex.printStackTrace();
                            greenwaveDTO.setControlOptTimes(m.getControlOptTimes());
                        }
                    }else {
                        greenwaveDTO.setControlOptTimes(m.getControlOptTimes());
                    }
                    greenwaveList.add(greenwaveDTO);
                });
                induceDTO.setGreenwaveList(greenwaveList);
                //获取关联模板信息
                //induceDTO.setInduceTemplateList(induceTemplateList.stream().filter(z -> Objects.isNull(z.getEquipCode()) || z.getEquipCode().equals(o.getEquipCode())).collect(Collectors.toList()));
                list.add(induceDTO);
            });

            jsonView.success(list);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("{} getByWhere error，jsonStr:{}", this.getClass().getSimpleName(), jsonStr, e);
        }
        return jsonView;
    }

    /**
     * 根据id查询记录
     *
     * @param id
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据id查询记录", notes = "根据id查询记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON)
    JsonViewObject getById(@ApiParam(value = "记录的id", required = true, example = "1") @PathVariable("id") @NotBlank(message = "查询id不能为空") String id){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            GreenwaveInduces GreenwaveInduces = this.greenwaveInducesService.getById(id);
            jsonView.success(GreenwaveInduces);
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("GET_FAILED_MSG"));
            log.error("AbstractRestServerImpl getById error, id:{}", id, e);
        }
        return jsonView;
    }

    /**
     * 根据id删除
     *
     * @param ids
     * @return JsonViewObject
     */
    @ApiOperation(value = "根据多个id删除记录", notes = "根据多个id删除记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON)
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "ids", dataType = "String", required = true, value = "多个记录id，用逗号分隔", example = "1,2")
    })
    @GetMapping(value = "/deleting", produces = MediaType.APPLICATION_JSON)
    JsonViewObject deleteByIds(@QueryParam("ids") @NotBlank(message = "删除ids不能为空") String ids){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        String[] idArray = ids.split(",");
        try {
            if (idArray.length > 0) {
                for (String id : idArray) {
                    jsonView = this.greenwaveInducesService.removeByIds(Arrays.asList(ids.split(",")))?jsonView.success():jsonView.fail();;
                }
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("DELETE_FAILED_MSG"));
            log.error("AbstractRestServerImpl deleteByIds error, id:{}", ids, e);
        }
        return jsonView;
    }

    /**
     * 新建记录
     *
     * @param greenwaveInduces
     * @return JsonViewObject
     */
    @ApiOperation(value = "新建记录", notes = "新建记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/creating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject save(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Create.class}) GreenwaveInduces greenwaveInduces){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();

        try {
            if (greenwaveInduces != null) {
                jsonView = this.greenwaveInducesService.saveOrUpdate(greenwaveInduces)?jsonView.success():jsonView.fail();
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("SAVE_FAILED_MSG"));
            log.error("AbstractRestServerImpl save error, jsonStr:{}", JSON.toJSONString(greenwaveInduces), e);
        }
        return jsonView;
    }

    /**
     * 修改记录
     *
     * @param greenwaveInduces
     * @return
     */
    @ApiOperation(value = "修改记录", notes = "修改记录", response = JsonViewObject.class, produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/updating", produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    JsonViewObject update(@ApiParam(value = "记录的JSON格式字符串", required = true) @RequestBody @Validated({ValidationGroups.Update.class}) GreenwaveInduces greenwaveInduces){
        JsonViewObject jsonView = JsonViewObject.newInstance();
        long start=System.currentTimeMillis();
        try {
            if (greenwaveInduces != null) {
                jsonView = this.greenwaveInducesService.saveOrUpdate(greenwaveInduces)?jsonView.success():jsonView.fail();
            }
        } catch (Exception e) {
            jsonView.fail(I18nResourceBundle.getConstants("UPDATE_FAILED_MSG"));
            log.error("AbstractRestServerImpl update error, jsonStr:{}", JSON.toJSONString(greenwaveInduces), e);
        }
        return jsonView;
    }
 
}
